Unit NLDDexImage;

Interface

Uses
  Windows, SysUtils, Classes, Controls, Graphics, Forms, Messages;

Type
  TNLDDexImageStyle     = (stDefault, stCenter, stTile);
  TNLDDexImageScaleMode = (smNormal, smScale, smStretch, smResize);

  TNLDDexImage = class(TGraphicControl) //TGraphicControl
  Private
    { Start Private declarations }
    FImageStyle        : TNLDDexImageStyle;
    FImageScaleMode    : TNLDDexImageScaleMode;
    FImageScaleRatio   : Single;
    FPicture           : TPicture;
    FImageTransparent  : Boolean;
    FDrawing           : Boolean;
    Procedure SetPicture(const Value: TPicture);
    Procedure SetImageStyle(Value: TNLDDexImageStyle);
    Procedure SetImageScaleRatio(Value: Single);
    Procedure SetImageScaleMode(Value: TNLDDexImageScaleMode);
    Procedure SetImageTransparent(Value: Boolean);
    Procedure PictureChanged(Sender: TObject);
    { End Private declarations }
  Protected
    { Start Protected declarations }
    Function DestRect: TRect;
    Function DoPalleteChange: Boolean;
    Procedure Paint; override;
    { End Protected declarations }
  Public
    { Start Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Destructor  Destroy; override;
    { End Public declarations }
  Published
    { Published declarations }
    { Default declarations }
    property Align;
    property Anchors;
    property AutoSize;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    { TDexImage declarations }
    Property Style        : TNLDDexImageStyle Read FImageStyle Write SetImageStyle Default stDefault;
    Property ScaleRatio   : Single Read FImageScaleRatio Write SetImageScaleRatio;
    Property ScaleMode    : TNLDDexImageScaleMode Read FImageScaleMode Write SetImageScaleMode Default smNormal;
    Property Picture      : TPicture Read FPicture Write SetPicture;
    Property Transparent  : Boolean Read FImageTransparent Write SetImageTransparent Default False;
  End;

Implementation

//============================================================================================================================
//Construct / Create the Component
Constructor TNLDDexImage.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner); { Always call the inherited. }
  FPicture          := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FImageStyle       := stDefault;
  FImageScaleMode   := smNormal;
  FImageScaleRatio  := 1;
End;
//============================================================================================================================

//============================================================================================================================
Destructor TNLDDexImage.Destroy;
Begin
  FreeAndNil(FPicture);
  Inherited Destroy; { Always call the inherited. }
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.Paint();
Var
  //Default
  SaveFDrawing : Boolean;
  //Style stTile
  TileRect : TRect;
  Y, X, cWidth, cHeight, pWidth, pHeight : Integer;
Begin
//Draw an nice rectangle around the image placeholder.
If csDesigning in ComponentState Then
Begin
  Canvas.Pen.Style := psDash;
  Canvas.Brush.Style := bsClear;
  Canvas.Rectangle(0, 0, Width, Height);
End;

SaveFDrawing := FDrawing;
FDrawing     := True;

Try
If Picture.Graphic <> NIL Then
Begin
  With Canvas Do
  Begin
    If Style = stTile Then
    Begin
      TileRect := DestRect;
      cWidth  := ClientWidth;
      cHeight := ClientHeight;
      pWidth  := TileRect.Right;
      pHeight := TileRect.Bottom;

    If (pWidth > 0) and (pHeight > 0) Then
    Begin
      Y := 0;
      While Y < cHeight Do
      Begin
        X := 0;
        While X < cWidth Do
        Begin
          TileRect.Left   := X;
          TileRect.Right  := (pWidth + TileRect.Left);
          Canvas.StretchDraw(TileRect, Picture.Graphic);
          Inc(X, pWidth);
        End;
        Inc(Y, pHeight);
        TileRect.Top    := Y;
        TileRect.Bottom := (pHeight + TileRect.Top);
      End;
    End;
    End
    Else
    Begin
      StretchDraw(DestRect, Picture.Graphic);
    End;
  End;
End;  
Finally
  FDrawing := SaveFDrawing;
End;
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.SetPicture(const Value: TPicture);
Begin
  FPicture.Assign(Value);
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.SetImageScaleMode(Value: TNLDDexImageScaleMode);
Begin
  If FImageScaleMode <> Value Then
  Begin
    FImageScaleMode := Value;
    PictureChanged(Self);
  End;
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.SetImageScaleRatio(Value: Single);
Begin
  If FImageScaleRatio <> Value Then
  Begin
    FImageScaleRatio := Value;
    PictureChanged(Self);
  End;
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.SetImageStyle(Value: TNLDDexImageStyle);
Begin
 If FImageStyle <> Value Then
 Begin
   FImageStyle := Value;
   PictureChanged(Self);
 End;
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.SetImageTransparent(Value: Boolean);
Begin
  If FImageTransparent <> Value Then
  Begin
    FImageTransparent := Value;
    PictureChanged(Self);
  End;
End;
//============================================================================================================================

//============================================================================================================================
Function TNLDDexImage.DestRect: TRect;
Var
  pWidth, pHeight, cWidth, cHeight: Integer;
  xyAspect: Double;
Begin
  //Get Width and Height information.
  pWidth  := Picture.Width;
  pHeight := Picture.Height;
  cWidth  := ClientWidth;
  cHeight := ClientHeight;

  //Start DestRect.
  If (ScaleMode = smStretch) or ((ScaleMode = smResize) and ((pWidth > cWidth) or (pHeight > cHeight))) Then
  Begin
    If (ScaleMode = smResize) and (pWidth > 0) and (pHeight > 0) Then
    Begin
      xyAspect := pWidth / pHeight;
      //If Width if greater-then Height
      If pWidth > pHeight Then
      Begin
        pWidth  := cWidth;
        pHeight := Trunc(cWidth / xyAspect);
        //Check if Picture Height isn't greater then Client Height Cus that would be to large.
        If pHeight > cHeight Then
        Begin
          pHeight := cHeight;
          pWidth  := Trunc(cHeight * xyAspect);
        End;
      End
      Else
      Begin
        pHeight := cHeight;
        pWidth  := Trunc(cWidth / xyAspect);
      End;
    End
    Else
    Begin
      pWidth  := cWidth;
      pHeight := cHeight;
    End;
  End;

  If ScaleMode = smScale Then
  Begin
    pWidth  := Round(pWidth * ScaleRatio);
    pHeight := Round(pHeight * ScaleRatio);
  End;

  With Result Do
  Begin
    Left   := 0;
    Top    := 0;
    Right  := pWidth;
    Bottom := pHeight;
  End;

  If Style = stCenter Then
  Begin
    OffsetRect(Result, ((cWidth - pWidth) div 2), ((cHeight - pHeight) div 2));
  End;
End;
//============================================================================================================================

//============================================================================================================================
Function TNLDDexImage.DoPalleteChange: Boolean;
Var
  ParentForm : TCustomForm;
  tmpGraphic : TGraphic;
Begin
  Result := False;
  tmpGraphic := Picture.Graphic;

  If Visible and (not (csLoading in ComponentState)) and (tmpGraphic <> NIL) and (tmpGraphic.PaletteModified) Then
  Begin
    If tmpGraphic.Palette = 0 Then
    Begin
      tmpGraphic.PaletteModified := False;
    End{Palette}
    Else
    Begin
      ParentForm := GetParentForm(Self);

      If Assigned(ParentForm) and ParentForm.Active and ParentForm.HandleAllocated Then
      Begin
        ParentForm.Perform(wm_QueryNewPalette, 0, 0);
      End
      Else
      Begin
        PostMessage(ParentForm.Handle, wm_QueryNewPalette, 0, 0);
      End;{Assigned}

      Result := True;
      tmpGraphic.PaletteModified := False;
    End;{Palette}
  End;{Visible}
End;
//============================================================================================================================

//============================================================================================================================
Procedure TNLDDexImage.PictureChanged(Sender: TObject);
Var
  dGraphic : TGraphic;
  DRect    : TRect;
Begin
  //Check Graphic State.
  dGraphic := Picture.Graphic;

  If dGraphic <> NIL Then
  Begin
    If not ((dGraphic is TMetaFile) or (dGraphic is TIcon)) Then
    Begin
      dGraphic.Transparent := FImageTransparent;
    End;

    DRect := DestRect;

    //Set Transparent
    If (not dGraphic.Transparent) and (DRect.Left <= 0) and (DRect.Top <= 0) and (DRect.Right >= Width) and (DRect.Bottom >= Height) Then
    Begin
      ControlStyle := ControlStyle + [csOpaque];
    End
    Else
    Begin
      ControlStyle := ControlStyle - [csOpaque];
    End;

    If DoPalleteChange and FDrawing Then
    Begin
      Update;
    End;
  End
  Else
  Begin
    ControlStyle := ControlStyle - [csOpaque];
  End;
  If not FDrawing Then
  Begin
    Invalidate;
  End;
End;
//============================================================================================================================
//============================================================================================================================
End.
